#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

/* Fork & Exec */
#include <spawn.h>
#include <sys/types.h>
#include <unistd.h>

/* Directory parsing */
#include <sys/types.h>
#include <dirent.h>

pid_t g_MPlayer_PID;	/* lets us kill our old copies of mplayer */

void PrintMenu(void)
{
	puts("Menu Options\n");
	puts("1. Play CD");
	puts("2. Stop CD");
	puts("3. Play MP3s");
	puts("4. Mute");
	puts("5. Unmute");
	puts("9. Quit");
}

int GetUserOption(void)
{
char opt[16];

	fgets(opt, sizeof(opt), stdin);
	opt[sizeof(opt)-1] = '\0';
	return atoi(opt);
}

pid_t ForkAndExec(const char *pName, char *ppArgs[])
{
pid_t process_id;

	/* We use vfork instead of fork because it's quicker */
	/* when you only intend the child to use execvp */
	process_id = vfork(); 
 
	switch(process_id) 
	{
	case	-1:
		printf("ERROR! Could not spawn %s...", pName);
		return -1;

	case	0:
		/* The child is born to die, having been replaced */
		/* by another program. */
		execvp(pName, ppArgs);
		exit(0);

	default:
		/* The parent process */
	}

	return process_id;
}

void PlayCD(void)
{
char *args[] = { "cdcd", "play", 0, };

	ForkAndExec("/usr/bin/cdcd", args);
}

void StopCD(void)
{
char *args[] = { "cdcd", "stop", 0, };

	ForkAndExec("/usr/bin/cdcd", args);
}

void StopMP3s(void)
{
	/* This is safe because you can't have a PID of 0 */
	if (g_MPlayer_PID)
	{
		kill(g_MPlayer_PID, SIGKILL);
		g_MPlayer_PID = 0;
	}
}

void PlayMP3Directory(const char *pDirname)
{
#define MAX_MP3_ENTRIES	128

DIR *pDir;
struct dirent *pDirEntry;
char *args[MAX_MP3_ENTRIES+3];	/* for simplicity, set a limit. */
int count, len;
char *pExt;

	StopMP3s();

	args[0] = "mplayer";
	args[1] = "-really-quiet";	/* Try to limit the output */

	pDir = opendir(pDirname);
	
	if (pDir == NULL)
	{
		return;
	}

	count = 2;
	while(count < MAX_MP3_ENTRIES)
	{
		pDirEntry = readdir(pDir);
		
		if (pDirEntry == NULL)		break;		/* end of dir */
		
		pExt = strrchr(pDirEntry->d_name, '.');
		if (pExt == NULL)		continue;	/* no ext */
		if (strcmp(pExt, ".mp3") != 0)	continue;	/* not mp3 */


		len = strlen(pDirname);
		len+= 1;	/* directory dividing slash */
		len+= strlen(pDirEntry->d_name);
		len+= 1;	/* NULL terminator */

		args[count] = malloc(len);
		if (args[count] == NULL)	break;		/* no mem */

		sprintf(args[count], "%s/%s", pDirname,  pDirEntry->d_name);
		/*puts(args[count]);*/
		count++;
	}

	closedir(pDir);

	args[count++] = 0;	/* terminator */

	g_MPlayer_PID = ForkAndExec("/usr/local/bin/mplayer", args);

	/* Now free that memory */
	count -= 2;	/* step back, past the terminator */
	do
	{
		free(args[count]);
	}
	while(--count > 1);	/* don't try & free the 1st two args */
}

	
void SetMixer(int iVolume)
{
char *args[] = { "aumix", "-d", "/dev/mixer0", "-v0", 0, };
char vol[8];

	/* Check volume, to stop sprintf overwriting buffer */
	if (iVolume < 0 || iVolume > 99)
	{
		return;
	}

	sprintf(vol, "-v%d", iVolume);
	args[3] = &vol[0];
	ForkAndExec("/usr/bin/aumix", args);
}

void ProcessMenu(int iOption)
{
	switch(iOption)
	{
	case	1:
			PlayCD();
			break;
	case	2:
			StopCD();
			break;
	case	3:
			PlayMP3Directory("mp3s");
			break;
	case	4:
			SetMixer(0);
			break;
	case	5:
			SetMixer(99);
			break;
	}
}

void StopAll(void)
{
	StopCD();
	StopMP3s();

	SetMixer(99);
}

void CloseHandler(int iSignal)
{
	/* Clear signal. We're only interested in fatal exits */
	/* so we're not gonna remove vital SIG's */
	signal(iSignal, SIG_DFL);

	puts("Exiting...");
	fflush(stdout);
	StopAll();
	exit(0);
}

int main(int argc, char **argv)
{
int iOpt;

	signal(SIGINT, CloseHandler);
	signal(SIGQUIT, CloseHandler);

	puts("Multimedia Shell - v0.1\n");
	do
	{
		PrintMenu();
		iOpt = GetUserOption();
		ProcessMenu(iOpt);
	}
	while(iOpt != 9);

	StopAll();

	return 0;
}

